


#include "ObjMem.h"
#include "ObjectDef.h"
#include "StructureDef.h"
#include "Map.h"

#include "Hci.h"	//because of selectedPlayer

/* For gates screen */

#include "Gates.h"

void CloseGates(GATE * psGate, UDWORD x, UDWORD y, STRUCTURE *pStr);
void OpenGates(GATE *psGate, UDWORD x, UDWORD y, STRUCTURE *pStr);
BOOL AddGateDoor(GATE *psGate, int X, int Y);
BOOL GatesThreat(UDWORD worldX, UDWORD worldY, int player, int range, BOOL bBlockAllies);
BOOL PlayerUnitsInRange(UDWORD worldX, UDWORD worldY, int player, int range);

static FEATURE	*testFeat;

void GatesUpdate()
{

	int				i;
	int				enemy_range = 128;		//range to look for enemy units
	int				friendly_range = 80;	//range to look for friendly units
	BOOL			bBlockAllies = FALSE;
	STRUCTURE		*psCurr;
	GATE			*psGate;
	
	for(i=0;i<MAX_PLAYERS;i=i+1)
	{
		for (psCurr = apsStructLists[i]; psCurr != NULL; psCurr = psCurr->psNext)
		{
			/* If gates */
			if((psCurr->pStructureType->type == REF_H_GATE) || (psCurr->pStructureType->type == REF_V_GATE))
			{
				psGate = (GATE *) psCurr->pFunctionality;

				/* Only check if normal state, otherwise just keep it (set on state change) */
				if(psGate->LockState == GSS_LOCK_NORMAL)
				{
					bBlockAllies = FALSE;	//Allies can pass

					/* If allies not allowed to pass */
					if(psGate->LockAllyState == GSS_ALOCK_LOCKED)
						bBlockAllies = TRUE;

					/* Enemy near, must close */
					if(GatesThreat(psCurr->x, psCurr->y, psCurr->player, enemy_range, bBlockAllies))
					{
						/* If opened and can close, then close gates */
						if((!psGate->bClosed) && (SafeToClosePlayerGates(psCurr->x,psCurr->y,psCurr->player, friendly_range)))
							CloseGates(psGate,psCurr->x >> TILE_SHIFT, psCurr->y >> TILE_SHIFT, psCurr);		//Close
					}
					else	/* No threat, look if we can open the gate, if closed */
					{
						if(psGate->bClosed)		//If closed
							OpenGates(psGate, psCurr->x >> TILE_SHIFT, psCurr->y >> TILE_SHIFT, psCurr);
					}
				}
			}
		}
	}
}

/* ----------------------------------------------------- */
/* If there are no own droids passing the gate right now */
/* ----------------------------------------------------- */
BOOL SafeToClosePlayerGates(UDWORD worldX, UDWORD worldY, int player, int range)
{
	if(objectInRange((BASE_OBJECT *)apsDroidLists[player], worldX, worldY, range))
	{
		return FALSE;					//there are own units passing the gate, so can't close
	}
	return TRUE;						//safe to close it
}

BOOL PlayerUnitsInRange(UDWORD worldX, UDWORD worldY, int player, int range)	//x and y must be in word coords (tile * 128)
{
	if(objectInRange((BASE_OBJECT *)apsDroidLists[player], worldX, worldY, range))
	{
		return TRUE;					//units in range
	}
	return FALSE;						//no units found within the range
}

BOOL GatesThreat(UDWORD worldX, UDWORD worldY, int player, int range, BOOL bBlockAllies)
{
	int				i;

	/* Check all players */
	for(i=0; i < MAX_PLAYERS; i++)
	{
		if(i != player)				//skip the owner
		{
			/* If allies are not allowed to pass, or allowed and player is not an ally */
			if(bBlockAllies || (!bBlockAllies && (alliances[i][player] != ALLIANCE_FORMED)))
			{
				if(PlayerUnitsInRange(worldX, worldY, i, range))
				{
					return TRUE;		//report danger
				}
			}
		}
	}

	return FALSE;					//no enemy units found near the gate
}

void OpenGates(GATE *psGate, UDWORD x, UDWORD y, STRUCTURE *pStr)
{
	MAPTILE			*psTile;
	SDWORD			i;

	if(!psGate->bClosed)
	{
		dbg_console("OpenGates(): Gate already open");
		return;
	}
	
	psTile = mapTile(x, y);

	SET_TILE_NOTBLOCKING(psTile);

	/* Remove door*/
	if(!RemoveGateDoor(psGate))
	{
		dbg_console("OpenGates() - failed to open gates");
	}

	psGate->bClosed = FALSE;

	dbg_console("****Gates opened at %d, %d.", x, y);

	return;
}

void CloseGates(GATE * psGate, UDWORD x, UDWORD y, STRUCTURE *pStr)
{
	MAPTILE			*psTile;
	SDWORD			i;

	if(psGate->bClosed)
	{
		dbg_console("CloseGates(): Gate already closed");
		return;
	}
	
	psTile = mapTile(x, y);

	CLEAR_TILE_NOTBLOCKING(psTile);

	/* Add door */
	if(!AddGateDoor(psGate, x << TILE_SHIFT, y << TILE_SHIFT))
	{
		dbg_console("CloseGates() - failed to add gates");
	}


	psGate->bClosed = TRUE;

	dbg_console("****Gates closed at %d, %d.", x, y);

	return;
}

BOOL IsGateAtTile(int X, int Y)
{
	STRUCTURE	*psStruct = NULL;
	//GATE		*psGate = NULL;
	
	if(TILE_HAS_STRUCTURE(mapTile(X, Y)))
	{
		psStruct = getTileStructure(X, Y);
		if(psStruct->pStructureType->type == REF_H_GATE ||
			psStruct->pStructureType->type == REF_V_GATE)
		{
			return TRUE;
		}

	}

	return FALSE;
}

STRUCTURE * GetGateAtTile(int X, int Y)
{
	STRUCTURE	*psStruct = NULL;
	//GATE		*psGate = NULL;
	
	if(TILE_HAS_STRUCTURE(mapTile(X, Y)))
	{
		psStruct = getTileStructure(X, Y);
		if(psStruct->pStructureType->type == REF_H_GATE ||
			psStruct->pStructureType->type == REF_V_GATE)
		{
			return psStruct;
		}

	}

	return NULL;
}


STRUCTURE * GetGateAt(SDWORD wX, SDWORD wY)
{
	STRUCTURE *psCurr = NULL;
	int i;
	
	for(i=0;i<MAX_PLAYERS;i=i+1)
	{
		for (psCurr = apsStructLists[i]; psCurr != NULL; psCurr = psCurr->psNext)
		{
			/* If gates */
			if((psCurr->pStructureType->type == REF_H_GATE) || (psCurr->pStructureType->type == REF_V_GATE))
			{
				if((psCurr->x == wX) && (psCurr->y == wY))
				{
					return psCurr;
				}
				else if((psCurr->x == (wX - 128)) && (psCurr->y == wY))	//1 tile to the left
				{
					return psCurr;
				}
				else if((psCurr->x == (wX + 128)) && (psCurr->y == wY))	//1 tile to the right
				{
					return psCurr;
				}
				else if((psCurr->x == wX) && (psCurr->y == (wY - 128)))		//1 tile up
				{
					return psCurr;
				}
				else if((psCurr->x == wX) && (psCurr->y == (wY + 128)))		//1 tile down
				{
					return psCurr;
				}
			}
		}
	}

	return NULL;
/*
	if(TILE_HAS_STRUCTURE(mapTile(X, Y)))
	{
		psStruct = getTileStructure(X, Y);
	}
	else if(TILE_HAS_STRUCTURE(mapTile(X - 1, Y)))	//go left 1 tile
	{
		psStruct = getTileStructure(X - 1, Y);
	}
	else if(TILE_HAS_STRUCTURE(mapTile(X, Y - 1)))	//go down 1 tile
	{
		psStruct = getTileStructure(X, Y - 1);
	}

	if(psStruct != NULL)
	{
		if(psStruct->pStructureType->type == REF_H_GATE ||
			psStruct->pStructureType->type == REF_V_GATE)
		{
			return psStruct->pStructureType->type;
		}
	}

	return -1;		//Not a gate
	*/
}

BOOL AddGateDoor(GATE *psGate, int X, int Y)
{
	FEATURE				*pfDoorNew = NULL;
	FEATURE_STATS		*psStat = NULL;		//type of feature - door
	FEATURE_STATS		*psFeature = asFeatureStats;
	UDWORD				inc;

	/* NULLSTRUCTURE passes */
	/*
	if(psStruc == NULL)
	{
		DbgMsg("AddGateDoor() - psStruc NULL");
		return FALSE;
	}*/

	//psGate = (GATE *) psStruc->pFunctionality;

	if(psGate->pfDoor != NULL)
	{
		DbgMsg("AddGateDoor() - gate already has a door");
		return FALSE;
	}

	/* Find gate door feature stats */
	for(inc=0; inc < numFeatureStats; inc++, psFeature++)
	{
		if(psFeature->subType == FEAT_GATE_DOOR)
		{
			psStat = psFeature;
			break;
		}
	}

	if(psStat == NULL)
	{
		DbgMsg("AddGateDoor() - couldn't find FEAT_GATE_DOOR");
		return FALSE;
	}

	//psStat = (FEATURE_STATS *)(asFeatureStats + FEAT_GATE_DOOR);

	psGate->pfDoor = buildFeature(psStat,X, Y, FALSE);
	//removeFeature(pfDoor);
	if(psGate->pfDoor == NULL)
	{
		DbgMsg("AddGateDoor() - Failed to add door");
		return FALSE;
	}

	/* Rotate door for vertical gate */
	if(psGate->bVertical)
		psGate->pfDoor->direction = 90;

	return TRUE;
}

BOOL RemoveGateDoor(GATE *psGate)
{
	if(psGate->pfDoor)
	{
		removeFeature(psGate->pfDoor);
		psGate->pfDoor = NULL;		//Does this produce a memory leak?
	}
	else
	{
		dbg_console("RemoveGateDoor() - no gates");
		return FALSE;
	}
}

/* Set the state of a secondary order for a Factory, return FALSE if failed. */
BOOL setGateState(STRUCTURE *psStruct, MENU_GATE_OPTIONS option, MENU_GATE_STATE State)
{
	UDWORD		CurrState;
    GATE		*psGate;

	//dbg_console("setGateState(%d,%d)",option,State);

	if(psStruct == NULL)
	{
		DbgMsg("setGateState() - gate is NULL");
		return FALSE;
	}
	else
	{
		if (psStruct->pStructureType->type == REF_H_GATE || psStruct->pStructureType->type == REF_V_GATE)
		{
			psGate = (GATE *)psStruct->pFunctionality;

			switch (option) {

				/* Lock state: Default/Locked/Unlocked */
				case GT_LOCK_STATE:

					psGate->LockState = State;

					switch(State){
						case GSS_LOCK_FORCE_LOCK:
						{
							CloseGates(psGate, psStruct->x >> TILE_SHIFT, psStruct->y >> TILE_SHIFT, psStruct);
							break;
						}
						
						case GSS_LOCK_NORMAL:
						{
							OpenGates(psGate, psStruct->x >> TILE_SHIFT, psStruct->y >> TILE_SHIFT, psStruct);
							break;
						}

						case GSS_LOCK_FORCE_UNLOCK:
						{
							OpenGates(psGate, psStruct->x >> TILE_SHIFT, psStruct->y >> TILE_SHIFT, psStruct);
							break;
						}

						default:		/* Unknown state parameter */
						{
							DbgMsg("setGateState() - wrong state parameter for GSS_LOCK_FORCE_LOCK: %d, %d.", option, State);
							return FALSE;
							break;
						}
					}

					break;

				/* Lock state for allies */
				case GT_ALLY_LOCK_STATE:
					psGate->LockAllyState = State;

					switch(State)
					{
						case GSS_ALOCK_NORMAL:
						case GSS_ALOCK_LOCKED:
						{
							break;
						}

						default:		/* Unknown state parameter */
						{
							DbgMsg("setGateState() - wrong state parameter for GT_ALLY_LOCK_STATE: %d, %d.", option, State);
							return FALSE;
							break;
						}
					}

					break;

				default:	/* Unknown state */
				{
					DbgMsg("setGateState() - wrong state: %d, %d.", option, State);
					return FALSE;
					break;
				}
			}

			return TRUE;
		}
		else
		{
			DbgMsg("setGateState() - structure not a gate.");
			return FALSE;
		}
	}
}

/* Get the state of a Gate */
BOOL getGateState(STRUCTURE *psStruct, MENU_GATE_OPTIONS option, MENU_GATE_STATE *pState)
{
	GATE		*psGate;

    if (psStruct->pStructureType->type == REF_H_GATE || psStruct->pStructureType->type == REF_V_GATE)
    {
		switch (option)
		{
			case GT_LOCK_STATE:
			{
				*pState = ((GATE *)psStruct->pFunctionality)->LockState;
				break;
			}
			case GT_ALLY_LOCK_STATE:
			{
				*pState = ((GATE *)psStruct->pFunctionality)->LockAllyState;
				break;
			}
			default:
			{
				DbgMsg("getGateState() - wrong state.");
				*pState = 0;
				return FALSE;
			}
		}

		//dbg_console("getGateState(%d, %d)", option, *pState);
		return TRUE;
    }
	else
	{
		DbgMsg("getGateState() - structure not a gate.");
		return FALSE;
	}
}